@@ -4,9 +4,10 @@
 import importlib
 import logging
 import os
+import sys
 from functools import cached_property
 
-import yaml
+import json5
 
 logger = logging.getLogger(__name__)
 
@@ -21,65 +22,84 @@ class Singleton(object):
         return cls.registered_instances[cls]
 
 
-_logfmt = '%(asctime)s %(levelname)s [%(process)s,%(thread)s] %(name)s %(message)s'
+def _path_join(*paths):
+    path = os.path.join(*paths)
+    return os.path.abspath(path)
 
 
 class GlobalInterface(Singleton):
-    # default config and log format
-    _config = {}
-    _logfmt = _logfmt
+    # for envvar prefix and default conf paths, [a-z]+
+    primary_name = 'volcanic'
 
-    # envvar prefix and conf paths will depend on this
+    # for package dir, [a-z.]+
     package_name = 'volkanic'
 
+    meta_config = {
+        'src_depth': 0,
+        'filename': 'config.json5',
+    }
+
+    # default config and log format
+    default_config = {}
+    default_logfmt = \
+        '%(asctime)s %(levelname)s [%(process)s,%(thread)s] %(name)s %(message)s'
+
     @classmethod
     def _fmt_envvar_name(cls, name):
-        prefix = cls.package_name.replace('.', '')
-        return '{}_{}'.format(prefix, name).upper()
+        return '{}_{}'.format(cls.primary_name, name).upper()
+
+    @classmethod
+    def _get_conf_search_paths(cls):
+        """
+        Make sure this method can be called without arguments.
+        """
+        filename = cls.meta_config['filename']
+        tmpls = [
+            '/etc/{}/{}',
+            '/{}/{}',
+            '/data/local/{}/{}',
+            '/data/{}/{}',
+            os.path.expanduser('~/.{}/{}'),
+        ]
+        paths = [p.format(cls.primary_name, filename) for p in tmpls]
+        paths += [cls.under_project_dir(filename)]
+        return paths
 
     @classmethod
     def _locate_conf(cls):
-        """\
-        Get path to the config file.
-        (Override this method in your subclasses for your specific project.)
+        """
         Returns: (str) absolute path to config file
         """
         envvar_name = cls._fmt_envvar_name('confpath')
-        path = os.environ.get(envvar_name)
-        if path:
-            return path
-        name = cls.package_name.replace('.', '-')
-        paths = [
-            '/{}/config.yml'.format(name),
-            os.path.expanduser('~/.{}/config.yml'.format(name)),
-            cls.under_package_dir('../../config.yml'),
-        ]
-        for path in paths:
+        try:
+            return os.environ[envvar_name]
+        except KeyError:
+            pass
+        for path in cls._get_conf_search_paths():
             path = os.path.abspath(path)
             if os.path.exists(path):
                 return path
 
     @staticmethod
     def _parse_conf(path: str):
-        return yaml.safe_load(open(path))
+        return json5.load(open(path))
 
     @cached_property
     def conf(self) -> dict:
         path = self._locate_conf()
         if path:
+            print('GlobalInterface.conf, path', path, file=sys.stderr)
             user_config = self._parse_conf(path)
-            logger.info('GlobalInterface.conf, path %s', path)
         else:
             user_config = {}
-        config = dict(self._config)
+        config = dict(self.default_config)
         config.update(user_config)
-        for k in ['data_dir', 'resources_dir']:
-            config[k] = os.path.expanduser(config[k])
         return config
 
     def under_data_dir(self, *paths) -> str:
         dirpath = self.conf['data_dir']
-        return os.path.join(dirpath, *paths)
+        path = os.path.join(dirpath, *paths)
+        return os.path.abspath(path)
 
     @classmethod
     def under_package_dir(cls, *paths) -> str:
@@ -87,12 +107,20 @@ class GlobalInterface(Singleton):
         pkg_dir = os.path.split(mod.__file__)[0]
         if not paths:
             return pkg_dir
-        return os.path.join(pkg_dir, *paths)
+        path = os.path.join(pkg_dir, *paths)
+        return os.path.abspath(path)
 
     def under_resources_dir(self, *paths):
         dirpath = self.conf['resources_dir']
         return os.path.join(dirpath, *paths)
 
+    @classmethod
+    def under_project_dir(cls, *paths):
+        n = cls.meta_config.get('src_depth', 0)
+        n += len(cls.package_name.split())
+        paths = ['..'] * n + list(paths)
+        return cls.under_package_dir(*paths)
+
     @cached_property
     def jinja2_env(self):
         # noinspection PyPackageRequirements
@@ -103,8 +131,9 @@ class GlobalInterface(Singleton):
         )
 
     @classmethod
-    def setup_logging(cls, level=None):
+    def setup_logging(cls, level=None, fmt=None):
         if not level:
             envvar_name = cls._fmt_envvar_name('loglevel')
             level = os.environ.get(envvar_name, 'DEBUG')
-        logging.basicConfig(level=level, format=cls._logfmt)
+        fmt = fmt or cls.default_logfmt
+        logging.basicConfig(level=level, format=fmt)
